// Übersetzung mit 
// g++ -std=c++14 -o bindComparison_boost bindComparison_boost.cpp
// Die verwendeten Boost-Bibliotheken sind "Header-only"

#include <iostream>
#include <algorithm>

#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/array.hpp>
#include <boost/random.hpp>

class secret {
public:
  secret(int s) :secret_(s)
  { /* nothing */ }

  int value() const {
    return secret_;
  }

private:
  //secret() = delete;
  int secret_ = 0;
};

const std::size_t ARRAYSIZE=10;

int main(int argc, char **argv) {
  boost::random::mt19937 generator;
  boost::random::uniform_int_distribution<> wuerfel(1, 6); 
  
  boost::array<boost::shared_ptr<secret>,ARRAYSIZE> v_secrets;
  for(std::size_t pos=0; pos<ARRAYSIZE; pos++) {
    v_secrets.at(pos) = boost::shared_ptr<secret>(
            new secret(wuerfel(generator))
            );
  }
  
  // Sortierung nach den "secret"-Werten. boost::shared_ptr
  // wird automatisch berücksichtigt
  auto withBind = boost::bind(&secret::value,_1) < boost::bind(&secret::value,_2);
  std::sort(
	    v_secrets.begin(),
	    v_secrets.end(),
	    withBind
  );
  // Sortierung auch möglich mit Boost.Lambda oder Boost.Phoenix

  for(auto s: v_secrets) { 
      std::cout << s->value() << std::endl; 
  }
}
