// Übersetzung mit 
// g++ -std=c++14 -o bindComparison_std bindComparison_std.cpp

#include <iostream>
#include <algorithm>
#include <memory>
#include <array>
#include <functional>
#include <random>

class secret {
public:
  secret(int s) :secret_(s)
  { /* nothing */ }

  int value() const {
    return secret_;
  }

private:
  secret() = delete;
  int secret_ = 0;
};

const std::size_t ARRAYSIZE=10;

int main(int argc, char **argv) {
  // Ggf. fuer std::bind benötigt
  // using namespace std::placeholders;

  std::mt19937 generator;
  std::uniform_int_distribution<int> wuerfel(1,6);
  
  std::array<std::shared_ptr<secret>,ARRAYSIZE> v_secrets;
  for(std::size_t pos=0; pos<ARRAYSIZE; pos++) {
    v_secrets.at(pos) = std::shared_ptr<secret>(
            new secret(wuerfel(generator))
            );
  }

  // Sortierung nach den "secret"-Werten
  auto lambda = [](
          std::shared_ptr<secret> x, 
          std::shared_ptr<secret> y
          ) -> bool 
          {
              return x->value() < y->value();
          };
  std::sort(
          v_secrets.begin(), 
          v_secrets.end(),
          // std::bind(&secret::value, _1) < std::bind(&secret::value, _2) 
          // Vergleich nicht möglich
          lambda
          );

  for(auto s: v_secrets) { std::cout << s->value() << std::endl; }
}
